// package dein.package ;

import java.io.*;
import java.text.*;
import java.util.*;


/**
 * Programm zum laufenden Sichern von Dateien im spezifizierten Verzeichnis.<br/>
 * <br>
 * Es durchluft rekursiv das spezifizierte Verzeichnis und alle Unterverzeichnisse
 * und prft alle enthaltenen Dateien.<br/>

 /s oder /S fr nur einmaligen Durchlauf

 * <br>
 * Aufruf:<br>
 * java -cp &lt;pfad_zu_VersionSave.class&gt; VersionSave &lt;start_directory&gt;
 * </pre>
 * Dieses Programm sollte in dem Verzeichnis gestartet werden,
 * welches die Root der zu instrumentierenden Java-Dateien darstellt und
 * auf welches die CLASSPATH-Variable zeigt.
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker  www.???
 * @version $Id$
 */
public class XCopy
{

   /**
    * Merker fr Exit-Status ( 0 - keine Fehler )
    */
   private static int iExitStatus = 0;

   /**
    * Verzeichnis mit den Original-Dateien
    */
   private static String srcDir;

   /**
    * Verzeichnis mit den Sicherungskopien
    */
   private static String sicoDir;

   /**
    * Timestamp-Datumsformat fr {@link #getTimeStamp}
    */
   private static final SimpleDateFormat formatTimestamp = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss",
         Locale.US);

   /**
    * Main-Methode dieser Klasse. <br/>
    * Beginnt im aktuellen Verzeichnis mit der Suche nach Dateien.<br/>
    */
   public static void main(String[] args)
   {
      boolean bRun = true;

      while (bRun)
      {
         // String srcDir = "." ;

         if (args.length < 1)
         {
            System.out.println("Start mit java XCopy <srcDir> [ <sicoDir> ]");
            System.out.println("Wenn <sicoDir> fehlt, wird automatisch ein Verzeichnis <srcDir>-<TimeStamp> angelegt");
            System.exit(1);
         }
         srcDir = args[ 0 ];
         System.out.println("VersionSave Start in " + srcDir);

         // Prfen, ob nur einmal durchlaufen werden soll
         for (int i = 1; i < args.length; i++)
         {
            if ("/S".equals(args[i].toUpperCase()))
            {
               bRun = false; // nur einmal durchlaufen
            }
         }

         // mehr als ein Parameter vorhanden aber zweite Parameter ist nicht /s fr einmaligen durchlauf
         if (args.length > 1 // .
               && !"/S".equals(args[ 1 ].toUpperCase()))
         {
            // als Parameter bergebenes Verzeichnis benutzen
            sicoDir = args[ 1 ];
         }
         else
         {
            // Default-Sicherungs-Verzeichnis Anlegen
            String strSrcDirWithoutSeparator = cutRight(srcDir, File.separator);

            if (".".equals(strSrcDirWithoutSeparator))
            {
                // Punkt durch Arbeitsverzeichnis ersetzen
               strSrcDirWithoutSeparator = System.getProperty("user.dir");
               strSrcDirWithoutSeparator = cutRight(strSrcDirWithoutSeparator, File.separator);
            }

            String strDateSuffix = formatTimestamp.format(new Date());

            //sicoDir = strSrcDirWithoutSeparator + "_versionsave" + File.separator;
            sicoDir = strSrcDirWithoutSeparator  + "-" + strDateSuffix + File.separator;

            System.out.println("Verwenden Default-Sicherungs-Verzeichnis " + sicoDir);

            if (!(new File(sicoDir)).exists())
            {
               System.out.println("Anlegen Default-Sicherungs-Verzeichnis " + sicoDir);
               (new File(sicoDir)).mkdirs();
            }
         }

         System.out.println("Kopieren nach " + sicoDir);
         scanDir(srcDir);

         // Ausgabe Uhrzeit zur berwachumg Ttigkeit und Aktualitt
         System.out.println("Last run at " + formatTimestamp.format(new Date())
               + "--------------------------------------------------");

         if (bRun)
         {
            try
            {
               // Thread.sleep(10 * 60 * 1000); // 10 Minuten warten
               Thread.sleep(1 * 60 * 1000); // 1 Minute warten
            }// end try
            catch (Exception e)
            {
               e.printStackTrace();
            }// end catch
         }
      }// end while
   }// end method main

   /**
    * Methode zum Durchlaufen des aktuellen Verzeichnisses und
    * aller Unterverzeichnisse. <br/>
    * Ruft fr alle Dateien die Methode {@link #processFile} auf.
    *
    * @param dirPath Pfad des zu durchsuchenden Verzeichnisses
    */
   public static void scanDir(String dirPath)
   {
      System.out.println("scanDir: " + dirPath);

      // nicht das build-Verzeichnis
      // if ( dirPath.endsWith( "\\build" ) ) {
//      if (dirPath.equals("C:\\metal_spec_develop\\MetalSpec\\build"))
//      {
//         return;
//      }
      //nicht eclipse-metadata
      if (dirPath.endsWith("\\.metadata"))
      {
         return;
      }

      // nicht das javadoc-Verzeichnis
      if (dirPath.equals(".\\javadoc"))
      {
         return;
      }

      // Ermittle Liste Dateien
      File[] fileArr = new File(dirPath).listFiles();

      for (int i = 0; i < fileArr.length; i++)
      {
         if (fileArr[i].isDirectory())
         {
            scanDir(fileArr[i].getParent() + File.separator + fileArr[i].getName());
         }
         else
         { // ein File
            processFile(fileArr[i].getParent() + File.separator + fileArr[i].getName());
         }
      } // end for
   }// end method scanDir

   /**
    * Methode zum Abarbeiten aller Dateien, ausser Verzeichnisse. <br/>
    * Diese Methode wird von {@link #scanDir} fr jede Datei<br/>
    * aufgerufen.<br/>
    * Sie ruft fr jede <b>*.java</b>-Dateien die Methode
    * {@link #processJavaFile} auf.
    *
    * @param fileName Dateiname mit Pfad und Endung
    */
   public static void processFile(String fileName)
   {
      // keine bak-Dateien
      if (fileName.toLowerCase().endsWith(".bak"))
      {
         return;
      }
      // keine class-Dateien
      if (fileName.toLowerCase().endsWith(".class"))
      {
         return;
      }
      // keine dbf-Dateien
      if (fileName.toLowerCase().endsWith(".dbf"))
      {
         return;
      }
      // keine dbt-Dateien
      if (fileName.toLowerCase().endsWith(".dbt"))
      {
         return;
      }
      // keine fpt-Dateien
      if (fileName.toLowerCase().endsWith(".fpt"))
      {
         return;
      }
      // keine obj-Dateien
      if (fileName.toLowerCase().endsWith(".obj"))
      {
         return;
      }
      // keine ntx-Dateien
      if (fileName.toLowerCase().endsWith(".ntx"))
      {
         return;
      }
      // keine exe-Dateien
      if (fileName.toLowerCase().endsWith(".exe"))
      {
         return;
      }
      // keine sts-Dateien
      if (fileName.toLowerCase().endsWith(".sts"))
      {
         return;
      }
      // keine $$$-Dateien
      if (fileName.indexOf("$$$") > -1)
      {
         return;
      }
      // keine XPPERROR.LOG-Dateien
      if (fileName.indexOf("XPPERROR.LOG") > -1)
      {
         return;
      }
      // keine XPPFATAL.LOG-Dateien
      if (fileName.indexOf("XPPFATAL.LOG") > -1)
      {
         return;
      }

      /*
       // nicht msdb_classes.jar
       if (fileName.endsWith("msdb_classes.jar")) {
       return;
       }
       */

      // System.out.println("processFile: " + fileName);
      File originalFile = new File(fileName);
      String copyFileName = sicoDir + cutLeft(fileName, srcDir);

      // System.out.println("copyFileName " + copyFileName);
      File copyFile = new File(copyFileName);
      long originalFileTime = originalFile.lastModified();
      long copyFileTime = copyFile.lastModified();

      if (originalFileTime > copyFileTime)
      {
         System.out.println("create backup " + fileName);
         renameFileDateSuffix(copyFileName);
         copyFile(fileName, copyFileName);
      }
   }// end method processFile

   /**
    * Bak-File mit DateTime-Suffix anlegen . <br/>
    * Als Endung wird <b>.bak</b> angehngt.<br/>
    * Siehe {@link #strBakSuffixCl}
    *
    * @param strPaFileName von dieser Datei ein Bak-File anlegen
    * @return true wenn erfolgreich
    */
   public static boolean renameFileDateSuffix(String strPaFileName)
   {
      try
      {
         File fileOriginal = new File(strPaFileName);
         if ( ! fileOriginal.exists() )
         {
           return false;
         }
         String strDateSuffix = formatTimestamp.format(new Date(fileOriginal.lastModified()));
         String strDateSuffixFileName = strPaFileName + "-" + strDateSuffix;

         System.out.println(strDateSuffixFileName);
         File fileDateSuffix = new File(strDateSuffixFileName);
         boolean bOk = fileOriginal.renameTo(fileDateSuffix);

         return bOk;
      }
      catch (Exception e)
      {
         System.err.println("" + e);
         e.printStackTrace();
      }
      return false;
   }// end method writeBakFile

   /**
    * Datei kopieren
    *
    * @param source Name mit Pfad der zu kopierenden Datei
    * @param dest   Name mit Pfad des Kopierzieles
    */
   private static void copyFile(String source, String dest)
   {
      final int PUFF_SIZE = 65536; // Puffergroesse kann entsprechend Speicher angepasst werden
      byte[] puff;
      long lastModified;

      try
      {
         FileInputStream fis = new FileInputStream(source);
         File fo = new File(dest);

         if (fo.exists())
         {
            if (!fo.delete())
            { // Zieldatei erst mal loeschen
               String s = new String("copyFile " + source + " " + dest + " not available");

               System.err.println(s);
               throw new IOException(s);
            }
         }

         // fo.createNewFile();
         createNewFile(fo);

         FileOutputStream fos = new FileOutputStream(fo);

         while (fis.available() > 0)
         {
            if (fis.available() >= PUFF_SIZE)
            {
               puff = new byte[PUFF_SIZE];
            }
            else
            {
               puff = new byte[fis.available()];
            }
            fis.read(puff);
            fos.write(puff);
         }
         fis.close();
         lastModified = (new File(source)).lastModified();
         fos.close();
         fo.setLastModified(lastModified); // beim Kopieren das nderungsdatum erhalten
      }
      catch (IOException ioX)
      {
         System.err.println(ioX.getMessage());
         System.err.println("copyFile " + source + " " + dest + " not available");
      }
   }// end method copyFile

   /**
    * Erzeugen einer neuen Datei . <br/>
    * Eine eventuell vorhandene gleichnamige Datei wird vorher gelscht.<br/>
    * Falls notwendig, wird das Verzeichnis fr die Datei angelegt.<br/>
    *
    * @param filePa anzulegende Datei
    * @throws IOException weiterwerfen Exception vom Erzeugen der Datei
    */
   private static void createNewFile(File filePa)
      throws IOException
   {
      if (filePa.exists())
      {
         filePa.delete();
      }

      try
      {
         filePa.createNewFile();
      }
      catch (IOException ioe)
      {
         // Versuch Verzeichnis anzulegen
         (new File(filePa.getParent())).mkdirs();
         // zweiter Versuch
         filePa.createNewFile();
      }// end catch
   }// end method createNewFile

   /**
    * Teilstring von String links abschneiden, wenn mglich.
    *
    * @param srcStr Original-String
    * @param cutStr abzuschneidender String
    * @return abgeschnittener String
    */
   private static String cutLeft(String srcStr, String cutStr)
   {
      if (srcStr == null || cutStr == null)
      {
         return srcStr;
      }

      String  retStr = srcStr;

      if (retStr.length() >= cutStr.length() && retStr.startsWith(cutStr))
      {
         retStr = retStr.substring(cutStr.length()); // cutStr abschneiden
      }

      return retStr;
   }// end method cutLeft

   /**
    * Teilstring von String rechts abschneiden, wenn mglich
    *
    * @param srcStr Original-String
    * @param cutStr abzuschneidender String
    * @return abgeschnittener String
    */
   public static String cutRight(String srcStr, String cutStr)
   {
      if (srcStr == null || cutStr == null)
      {
         return srcStr;
      }

      String  retStr = srcStr;

      if (retStr.length() > cutStr.length() && retStr.endsWith(cutStr))
      {
         retStr = retStr.substring(0, retStr.length() - cutStr.length());  // cutStr abschneiden
      }

      return retStr;
   } // end method cutRight

}// end class VersionSave
